"""
    Patch a given product, or list of products as specified.
"""
__version__ = '4.0.0'

import Globals

def applyPatch(module):
    """ Given a patch module (Not the NAME, the MODULE)
        apply the defined patches

        Patch modules must contain at least:
         .targets             (List/Tuple of products to patch)
         .manage_options      (List/Tuple of manage_options to add)
         .backup_methods      (Dictionary mapping new names to old) (Optional)
         .methods             (Dictionary mapping names to methods)
    """
    if (hasattr(module, 'targets') and
        hasattr(module, 'manage_options') and
        hasattr(module, 'methods')):
        if (type(module.targets)!=type([]) and
            type(module.targets)!=type(())):
            module.targets = (module.targets,)
        if (type(module.manage_options)!=type([]) and
            type(module.manage_options)!=type(())):
            module.manage_options = (module.manage_options,)
        if not hasattr(module, 'backup_methods'):
            module.backup_methods = {}
        if type(module.backup_methods)!=type({}):
            raise "Patch Error", "'backup_methods' bad in module: %s" % module.__name__
        if type(module.methods)!=type({}):
            raise "Patch Error", "'methods' bad in module: %s" % module.__name__
        for target in module.targets:
            for method in module.backup_methods.items():
                if not hasattr(target, method[1]):
                    setattr(target, method[1], getattr(target, method[0]))
            for method in module.methods.items():
                setattr(target, method[0], method[1])
            newOptions = []
            if target.__dict__.has_key('manage_options'):
                for option in target.manage_options:
                    for newMo in module.manage_options:
                        if (not option.has_key('label') or
                            not option['label'] == newMo['label']):
                            newOptions.append(option)
            for newMo in module.manage_options:
                newOptions.append(newMo)
            target.manage_options = tuple(newOptions)
            Globals.default__class_init__(target)
